@@ -661,7 +661,9 @@ def get_db_fingerprint(conn=None, fname=None, with_dump=False, eol=None):
 		("SELECT setting FROM pg_settings WHERE name = 'server_version'", "Version (PG)"),
 		("SELECT setting FROM pg_settings WHERE name = 'server_encoding'", "Encoding (PG)"),
 		("SELECT setting FROM pg_settings WHERE name = 'lc_collate'", "LC_COLLATE (PG)"),
+		("SELECT datcollate FROM pg_database WHERE datname = current_database()", "pg_database.datcollate (PG)"),
 		("SELECT setting FROM pg_settings WHERE name = 'lc_ctype'", "LC_CTYPE (PG)"),
+		("SELECT datctype FROM pg_database WHERE datname = current_database()", "pg_database.datctype (PG)"),
 		("SELECT count(1) FROM dem.identity", "Patients"),
 		("SELECT count(1) FROM clin.encounter", "Contacts"),
 		("SELECT count(1) FROM clin.episode", "Episodes"),
@@ -696,9 +698,22 @@ def get_db_fingerprint(conn=None, fname=None, with_dump=False, eol=None):
 	except KeyError:
 		lines.append('%20s: %s' % ('Schema hash', md5_sum))
 	for cmd, label in queries:
-		curs.execute(cmd)
-		rows = curs.fetchall()
-		lines.append('%20s: %s' % (label, rows[0][0]))
+		try:
+			curs.execute(cmd)
+			rows = curs.fetchall()
+			if rows:
+				val = rows[0][0]
+			else:
+				val = '<not found>'
+		except PG_ERROR_EXCEPTION as pg_exc:
+			if pg_exc.pgcode != sql_error_codes.INSUFFICIENT_PRIVILEGE:
+				raise
+
+			if pg_exc.pgerror is None:
+				val = '[%s]: insufficient privileges' % pg_exc.pgcode
+			else:
+				val = '[%s]: %s' % (pg_exc.pgcode, pg_exc.pgerror)
+		lines.append('%20s: %s' % (label, val))
 	if with_dump:
 		lines.append('')
 		lines.append(str(get_schema_structure(link_obj = curs)))
@@ -713,6 +728,30 @@ def get_db_fingerprint(conn=None, fname=None, with_dump=False, eol=None):
 	outfile.close()
 	return fname
 
+#------------------------------------------------------------------------
+def user_needs_password_encryption_switch(user:str=None) -> bool:
+	args = {'usr': user}
+	if not user:
+		SQL = 'SELECT gm.user_needs_md5_2_scramsha256_pwd_switch(CURRENT_USER)'
+	else:
+		SQL = 'SELECT gm.user_needs_md5_2_scramsha256_pwd_switch(%(usr)s)'
+	rows, idx = run_ro_queries(queries = [{'cmd': SQL, 'args': args}])
+	if not rows[0][0]:
+		return False
+
+	_log.debug('user [%s] uses md5 password', user)
+	SQL = 'SELECT setting = %(val)s FROM pg_settings WHERE name = %(opt)s'
+	args = {
+		'val': 'scram-sha-256',
+		'opt': 'password_encryption'
+	}
+	rows, idx = run_ro_queries(queries = [{'cmd': SQL, 'args': args}])
+	if not rows[0][0]:
+		return False
+
+	_log.debug('PostgreSQL cluster configured for SCRAM-SHA256 password encryption, re-encryption of user password recommended')
+	return True
+
 #------------------------------------------------------------------------
 def get_current_user():
 	rows, idx = run_ro_queries(queries = [{'cmd': 'select CURRENT_USER'}])
@@ -2245,10 +2284,10 @@ def sanity_check_database_settings(hipaa:bool=False) -> (int, str):
 		'fsync': [['on'], 'data loss/corruption', True],
 		'full_page_writes': [['on'], 'data loss/corruption', False],
 		'lc_messages': [['C'], 'suboptimal error detection', False],
-		'password_encryption': [['on', 'md5', 'scram-sha-256'], 'breach of confidentiality', False],
-		#u'regex_flavor': [[u'advanced'], u'query breakage', False],					# 9.0 doesn't support this anymore, default now advanced anyway
+		'password_encryption': [['scram-sha-256'], 'breach of confidentiality', False],
+		#u'regex_flavor': [[u'advanced'], u'query breakage', False],				# hardwired in PG9+
 		'synchronous_commit': [['on'], 'data loss/corruption', False],
-		'sql_inheritance': [['on'], 'query breakage, data loss/corruption', True],	# IF returned (<PG10): better be ON, if NOT returned (PG10): hardwired
+		#'sql_inheritance': [['on'], 'query breakage, data loss/corruption', True],	# hardwired in PG10+
 		'ignore_checksum_failure': [['off'], 'data loss/corruption', False],		# starting with PG 9.3
 		'track_commit_timestamp': [['on'], 'suboptimal auditing', False],			# starting with PG 9.3
 	}
@@ -2848,6 +2887,17 @@ SELECT to_timestamp (foofoo,'YYMMDD.HH24MI') FROM (
 		with open('x-pg_temp_func.txt', 'w', encoding = 'utf8') as f:
 			f.write(__get_schema_structure_by_pg_temp_func())
 
+	#--------------------------------------------------------------------
+	def test_user_needs_password_encryption_switch(user:str=None):
+		users = [
+			sys.argv[2],
+			'any-staff',
+			None
+		]
+		request_login_params(setup_pool = True)
+		for user in users:
+			print('user [%s]' % user, user_needs_password_encryption_switch(user))
+
 	#--------------------------------------------------------------------
 	# run tests
 
@@ -2880,7 +2930,8 @@ SELECT to_timestamp (foofoo,'YYMMDD.HH24MI') FROM (
 	#test_get_db_fingerprint()
 	#test_schema_compatible()
 	#test_get_schema_structure()
-	test___get_schema_structure()
+	#test___get_schema_structure()
 	#test_pg_temp_concat()
+	test_user_needs_password_encryption_switch()
 
 # ======================================================================
